﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking;

namespace HeuristicLab.Encodings.PackingEncoding.PackingSequence {
  [Item("Sequence Move Maker", "Peforms a Sequence move on a given Sequence vector and updates the quality.")]
  [StorableClass]
  public class SequenceMoveMaker : SingleSuccessorOperator, IMoveMaker, IPackingSequenceMoveOperator {
    public override bool CanChangeName {
      get { return false; }
    }
    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Quality"]; }
    }
    public ILookupParameter<DoubleValue> MoveQualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["MoveQuality"]; }
    }
    public ILookupParameter<IPackingMove> PackingMoveParameter {
      get { return (ILookupParameter<IPackingMove>)Parameters["PackingMove"]; }
    }
    public ILookupParameter<PackingSequenceEncoding> PackingSequenceParameter {
      get { return (ILookupParameter<PackingSequenceEncoding>)Parameters["PackingSequence"]; }
    }
    public ILookupParameter<IPackingPlan> PackingPlanParameter {
      get { return (ILookupParameter<IPackingPlan>)Parameters["PackingPlan"]; }
    }
    public ILookupParameter<IPackingPlan> PackingPlanAfterMoveParameter {
      get { return (ILookupParameter<IPackingPlan>)Parameters["PackingPlanAfterMove"]; }
    }

    [StorableConstructor]
    protected SequenceMoveMaker(bool deserializing) : base(deserializing) { }
    protected SequenceMoveMaker(SequenceMoveMaker original, Cloner cloner) : base(original, cloner) { }
    public SequenceMoveMaker()
      : base() {
      Parameters.Add(new LookupParameter<DoubleValue>("Quality", "The quality of the solution."));
      Parameters.Add(new LookupParameter<IPackingMove>("PackingMove", "The move to evaluate."));
      Parameters.Add(new LookupParameter<DoubleValue>("MoveQuality", "The relative quality of the move."));
      Parameters.Add(new LookupParameter<PackingSequenceEncoding>("PackingSequence", "The solution as Sequence vector."));
      Parameters.Add(new LookupParameter<IPackingPlan>("PackingPlan", "The currently best performing, decoded bin-packing solution represented as generalized packing-plan."));
      Parameters.Add(new LookupParameter<IPackingPlan>("PackingPlanAfterMove", "The moved and decoded bin-packing solution represented as generalized packing-plan."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SequenceMoveMaker(this, cloner);
    }

    public override IOperation Apply() {
      IPackingMove move = PackingMoveParameter.ActualValue;
      PackingSequenceEncoding PackingSequence = PackingSequenceParameter.ActualValue;
      DoubleValue moveQuality = MoveQualityParameter.ActualValue;
      DoubleValue quality = QualityParameter.ActualValue;

      PackingSequence.PackingSequence = (move.GetSolutionAfterMove() as PackingSequenceEncoding).PackingSequence;
      PackingPlanParameter.ActualValue = PackingPlanAfterMoveParameter.ActualValue;

      quality.Value = moveQuality.Value;

      return base.Apply();
    }
  }
}
